@@ -4,19 +4,16 @@ import uuid
 
 import numpy as np
 
-try:
-    # pip install pycollada
-    import collada
-except BaseException:
-    collada = None
-
 from .. import util
 from .. import visual
 
 from ..constants import log
 
 
-def load_collada(file_obj, resolver=None, **kwargs):
+def load_collada(file_obj,
+                 resolver=None,
+                 ignore_broken=False,
+                 **kwargs):
     """
     Load a COLLADA (.dae) file into a list of trimesh kwargs.
 
@@ -26,6 +23,10 @@ def load_collada(file_obj, resolver=None, **kwargs):
       Containing a COLLADA file
     resolver : trimesh.visual.Resolver or None
       For loading referenced files, like texture images
+    ignore_broken: bool
+      Ignores broken references during loading:
+        [collada.common.DaeUnsupportedError,
+         collada.common.DaeBrokenRefError]
     kwargs : **
       Passed to trimesh.Trimesh.__init__
 
@@ -37,7 +38,11 @@ def load_collada(file_obj, resolver=None, **kwargs):
     import collada
 
     # load scene using pycollada
-    c = collada.Collada(file_obj)
+    c = collada.Collada(
+        file_obj,
+        ignore=[collada.common.DaeUnsupportedError,
+                collada.common.DaeBrokenRefError]
+        if ignore_broken else None)
 
     # Create material map from Material ID to trimesh material
     material_map = {}
@@ -198,7 +203,7 @@ def _parse_node(node,
                     color = s['COLOR'][0][4].data
                     color_index = primitive.index[:, :, s['COLOR'][0][0]]
                     colors = color[color_index].reshape(
-                        len(color_index) * 3, 3)
+                        len(color_index) * 3, -1)
 
                 faces = np.arange(
                     vertices.shape[0]).reshape(
@@ -319,7 +324,7 @@ def _parse_material(effect, resolver):
     if (effect.transparent is not None
             and not isinstance(effect.transparent, collada.material.Map)):
         baseColorFactor = tuple(
-            np.append(baseColorFactor[:3], float(int(255 * effect.transparent[3]))))
+            np.append(baseColorFactor[:3], float(effect.transparent[3])))
 
     return visual.material.PBRMaterial(
         emissiveFactor=emissiveFactor,
