@@ -1,3 +1,4 @@
+import hashlib
 import numpy as np
 import collections
 
@@ -16,7 +17,7 @@ from ..parent import Geometry3D
 from . import cameras
 from . import lighting
 
-from .transforms import TransformForest
+from .transforms import SceneGraph
 
 
 class Scene(Geometry3D):
@@ -59,7 +60,7 @@ class Scene(Geometry3D):
         self.geometry = collections.OrderedDict()
 
         # create a new graph
-        self.graph = TransformForest(base_frame=base_frame)
+        self.graph = SceneGraph(base_frame=base_frame)
 
         # create our cache
         self._cache = caching.Cache(id_function=self.md5)
@@ -77,7 +78,9 @@ class Scene(Geometry3D):
 
         self.camera = camera
         self.lights = lights
-        self.camera_transform = camera_transform
+
+        if camera is not None and camera_transform is not None:
+            self.camera_transform = camera_transform
 
     def apply_transform(self, transform):
         """
@@ -96,7 +99,8 @@ class Scene(Geometry3D):
                      node_name=None,
                      geom_name=None,
                      parent_node_name=None,
-                     transform=None):
+                     transform=None,
+                     extras=None):
         """
         Add a geometry to the scene.
 
@@ -108,17 +112,17 @@ class Scene(Geometry3D):
         ----------
         geometry : Trimesh, Path2D, Path3D PointCloud or list
           Geometry to initially add to the scene
-        base_frame : str or hashable
-          Name of base frame
-        metadata : dict
-          Any metadata about the scene
-        graph : TransformForest or None
-          A passed transform graph to use
+        node_name: Name of the added node.
+        geom_name: Name of the added geometry.
+        parent_node_name: Name of the parent node in the graph.
+        transform: Transform that applies to the added node.
+        extras: Optional metadata for the node.
 
         Returns
         ----------
         node_name : str
-          Name of node in self.graph
+          Name of single node in self.graph (passed in) or None if
+          node was not added (eg. geometry was null or a Scene).
         """
 
         if geometry is None:
@@ -131,11 +135,12 @@ class Scene(Geometry3D):
                 node_name=node_name,
                 geom_name=geom_name,
                 parent_node_name=parent_node_name,
-                transform=transform) for value in geometry]
+                transform=transform,
+                extras=extras) for value in geometry]
         elif isinstance(geometry, dict):
             # if someone passed us a dict of geometry
             for key, value in geometry.items():
-                self.add_geometry(value, geom_name=key)
+                self.add_geometry(value, geom_name=key, extras=extras)
             return
         elif isinstance(geometry, Scene):
             # concatenate current scene with passed scene
@@ -190,7 +195,9 @@ class Scene(Geometry3D):
                           frame_from=parent_node_name,
                           matrix=transform,
                           geometry=name,
-                          geometry_flags={'visible': True})
+                          geometry_flags={'visible': True},
+                          extras=extras)
+
         return node_name
 
     def delete_geometry(self, names):
@@ -224,24 +231,37 @@ class Scene(Geometry3D):
           MD5 hash of scene
         """
         # start with transforms hash
-        return util.md5_object(self._hashable())
+        return hashlib.md5(self._hashable()).hexdigest()
 
     def crc(self):
+        """
+        Get a CRC of the current geometry and graph information.
+
+        Returns
+        ---------
+        crc : int
+          Hash of current graph and geometry.
+        """
         return caching.crc32(self._hashable())
 
     def _hashable(self):
-        hashes = [self.graph.md5()]
-        for g in self.geometry.values():
-            if hasattr(g, 'md5'):
-                hashes.append(g.md5())
-            elif hasattr(g, 'tostring'):
-                hashes.append(str(hash(g.tostring())))
-            else:
-                # try to just straight up hash
-                # this may raise errors
-                hashes.append(str(hash(g)))
-        hashable = ''.join(sorted(hashes)).encode('utf-8')
-        return hashable
+        """
+        Return information about scene which is hashable.
+
+        Returns
+        ---------
+        hashable : str
+          Data which can be hashed.
+        """
+        # start with the last modified time of the scene graph
+        hashable = [self.graph.modified()]
+        # crc is an abstractmethod for all Geometry3D
+        # objects so everything should really have it
+        hashable.extend(str(i.crc()) for i in
+                        self.geometry.values()
+                        if hasattr(i, 'crc'))
+        # crc requires bytes so encode to utf-8
+        return ':'.join(sorted(hashable)).encode('utf-8')
 
     @property
     def is_empty(self):
@@ -292,7 +312,8 @@ class Scene(Geometry3D):
 
         Returns
         -----------
-        corners: (n, 3) float, points in space
+        corners: (n, 3) float
+          Points in space
         """
         # the saved corners of each instance
         corners_inst = []
@@ -470,29 +491,35 @@ class Scene(Geometry3D):
 
         Returns
         -----------
-        duplicates : (m) sequenc
-          Keys of self.nodes that represent identical geometry
+        duplicates : (m) sequence
+          Keys of self.graph that represent identical geometry
         """
         # if there is no geometry we can have no duplicate nodes
         if len(self.geometry) == 0:
             return []
 
         # geometry name : md5 of mesh
-        mesh_hash = {k: int(m.identifier_md5, 16)
-                     for k, m in self.geometry.items()}
-        # the name of nodes in the scene graph with geometry
-        node_names = np.array(self.graph.nodes_geometry)
-        # the geometry names for each node in the same order
-        node_geom = np.array([self.graph[i][1] for i in node_names])
-        # the mesh md5 for each node in the same order
-        node_hash = np.array([mesh_hash[v] for v in node_geom])
-        # indexes of identical hashes
-        node_groups = grouping.group(node_hash)
-        # sequence of node names where each
-        # sublist has identical geometry
-        duplicates = [np.sort(node_names[g]).tolist()
-                      for g in node_groups]
-        return duplicates
+        hashes = {k: int(m.identifier_md5, 16)
+                  for k, m in self.geometry.items()
+                  if hasattr(m, 'identifier_md5')}
+
+        # bring into local scope for loop
+        graph = self.graph
+        # get a hash for each node name
+        # scene.graph node name : hashed geometry
+        node_hash = {node: hashes.get(
+            graph[node][1]) for
+            node in graph.nodes_geometry}
+
+        # collect node names for each hash key
+        duplicates = collections.defaultdict(list)
+        # use a slightly off-label list comprehension
+        # for debatable function call overhead avoidance
+        [duplicates[hashed].append(node) for node, hashed
+         in node_hash.items() if hashed is not None]
+
+        # we only care about the values keys are garbage
+        return list(duplicates.values())
 
     def deduplicated(self):
         """
@@ -575,7 +602,7 @@ class Scene(Geometry3D):
     @property
     def camera_transform(self):
         """
-        Get camera transform in the base frame
+        Get camera transform in the base frame.
 
         Returns
         -------
@@ -584,6 +611,18 @@ class Scene(Geometry3D):
         """
         return self.graph[self.camera.name][0]
 
+    @camera_transform.setter
+    def camera_transform(self, matrix):
+        """
+        Set the camera transform in the base frame
+
+        Parameters
+        ----------
+        camera_transform : (4, 4) float
+          Camera transform in the base frame
+        """
+        self.graph[self.camera.name] = matrix
+
     def camera_rays(self):
         """
         Calculate the trimesh.scene.Camera origin and ray
@@ -613,20 +652,6 @@ class Scene(Geometry3D):
                    transformations.translation_from_matrix(transform))
         return origins, vectors, pixels
 
-    @camera_transform.setter
-    def camera_transform(self, camera_transform):
-        """
-        Set the camera transform in the base frame
-
-        Parameters
-        ----------
-        camera_transform : (4, 4) float
-          Camera transform in the base frame
-        """
-        if camera_transform is None:
-            return
-        self.graph[self.camera.name] = camera_transform
-
     @property
     def camera(self):
         """
@@ -751,6 +776,38 @@ class Scene(Geometry3D):
 
         return np.array(result)
 
+    def subscene(self, node):
+        """
+        Get part of a scene that succeeds a specified node.
+
+        Parameters
+        ------------
+        node : any
+          Hashable key in `scene.graph`
+
+        Returns
+        -----------
+        subscene : Scene
+          Partial scene generated from current.
+        """
+        # get every node that is a successor to specified node
+        # this includes `node`
+        graph = self.graph
+        nodes = graph.transforms.successors(node)
+        # get every edge that has an included node
+        edges = [e for e in graph.to_edgelist()
+                 if e[0] in nodes]
+
+        # create a scene graph whet
+        graph = SceneGraph(base_frame=node)
+        graph.from_edgelist(edges)
+
+        geometry_names = set([e[2]['geometry'] for e in edges
+                              if 'geometry' in e[2]])
+        geometry = {k: self.geometry[k] for k in geometry_names}
+        result = Scene(geometry=geometry, graph=graph)
+        return result
+
     @caching.cache_decorator
     def convex_hull(self):
         """
@@ -806,10 +863,9 @@ class Scene(Geometry3D):
         png : bytes
           Render of scene as a PNG
         """
-        from ..viewer import render_scene
-        png = render_scene(scene=self,
-                           resolution=resolution,
-                           **kwargs)
+        from ..viewer.windowed import render_scene
+        png = render_scene(
+            scene=self, resolution=resolution, **kwargs)
         return png
 
     @property
@@ -928,7 +984,7 @@ class Scene(Geometry3D):
             else:
                 raise ValueError('explode vector wrong shape!')
 
-            transform[0:3, 3] += offset
+            transform[:3, 3] += offset
             self.graph[node_name] = transform
 
     def scaled(self, scale):
